package javax.swing.text.html;

import javax.swing.text.View;
import javax.swing.SizeRequirements;
import javax.swing.text.AttributeSet;
import java.util.StringTokenizer;
import javax.swing.text.Element;
import javax.swing.text.BoxView;

class FrameSetView extends BoxView
{
    String[] children;
    int[] percentChildren;
    int[] absoluteChildren;
    int[] relativeChildren;
    int percentTotals;
    int absoluteTotals;
    int relativeTotals;
    
    public FrameSetView(Element elem, int axis) {
        super(elem, axis);
        this.children = null;
    }
    
    private String[] parseRowColSpec(HTML.Attribute attribute) {
        AttributeSet attributes = this.getElement().getAttributes();
        String str = "*";
        if (attributes != null && attributes.getAttribute(attribute) != null) {
            str = (String)attributes.getAttribute(attribute);
        }
        StringTokenizer o = new StringTokenizer(str, ",");
        int countTokens = o.countTokens();
        String[] array = new String[Math.max(countTokens, this.getViewCount())];
        int i;
        for (i = 0; i < countTokens; ++i) {
            array[i] = o.nextToken().trim();
            if (array[i].equals("100%")) {
                array[i] = "*";
            }
        }
        while (i < array.length) {
            array[i] = "*";
            ++i;
        }
        return array;
    }
    
    private void init() {
        if (this.getAxis() == 1) {
            this.children = this.parseRowColSpec(HTML.Attribute.ROWS);
        }
        else {
            this.children = this.parseRowColSpec(HTML.Attribute.COLS);
        }
        this.percentChildren = new int[this.children.length];
        this.relativeChildren = new int[this.children.length];
        this.absoluteChildren = new int[this.children.length];
        for (int i = 0; i < this.children.length; ++i) {
            this.percentChildren[i] = -1;
            this.relativeChildren[i] = -1;
            this.absoluteChildren[i] = -1;
            if (this.children[i].endsWith("*")) {
                if (this.children[i].length() > 1) {
                    this.relativeChildren[i] = Integer.parseInt(this.children[i].substring(0, this.children[i].length() - 1));
                    this.relativeTotals += this.relativeChildren[i];
                }
                else {
                    this.relativeChildren[i] = 1;
                    ++this.relativeTotals;
                }
            }
            else if (this.children[i].indexOf(37) != -1) {
                this.percentChildren[i] = this.parseDigits(this.children[i]);
                this.percentTotals += this.percentChildren[i];
            }
            else {
                this.absoluteChildren[i] = Integer.parseInt(this.children[i]);
            }
        }
        if (this.percentTotals > 100) {
            for (int j = 0; j < this.percentChildren.length; ++j) {
                if (this.percentChildren[j] > 0) {
                    this.percentChildren[j] = this.percentChildren[j] * 100 / this.percentTotals;
                }
            }
            this.percentTotals = 100;
        }
    }
    
    @Override
    protected void layoutMajorAxis(int allocated, int n, int[] offsets, int[] spans) {
        if (this.children == null) {
            this.init();
        }
        SizeRequirements.calculateTiledPositions(allocated, null, this.getChildRequests(allocated, n), offsets, spans);
    }
    
    protected SizeRequirements[] getChildRequests(int n, int axis) {
        int[] array = new int[this.children.length];
        this.spread(n, array);
        int viewCount = this.getViewCount();
        SizeRequirements[] array2 = new SizeRequirements[viewCount];
        int i = 0;
        int n2 = 0;
        while (i < viewCount) {
            View view = this.getView(i);
            if (view instanceof FrameView || view instanceof FrameSetView) {
                array2[i] = new SizeRequirements((int)view.getMinimumSpan(axis), array[n2], (int)view.getMaximumSpan(axis), 0.5f);
                ++n2;
            }
            else {
                array2[i] = new SizeRequirements((int)view.getMinimumSpan(axis), (int)view.getPreferredSpan(axis), (int)view.getMaximumSpan(axis), view.getAlignment(axis));
            }
            ++i;
        }
        return array2;
    }
    
    private void spread(int i, int[] array) {
        if (i == 0) {
            return;
        }
        int n = i;
        for (int j = 0; j < array.length; ++j) {
            if (this.absoluteChildren[j] > 0) {
                array[j] = this.absoluteChildren[j];
                n -= array[j];
            }
        }
        int n2 = n;
        for (int k = 0; k < array.length; ++k) {
            if (this.percentChildren[k] > 0 && n2 > 0) {
                array[k] = this.percentChildren[k] * n2 / 100;
                n -= array[k];
            }
            else if (this.percentChildren[k] > 0 && n2 <= 0) {
                array[k] = i / array.length;
                n -= array[k];
            }
        }
        if (n > 0 && this.relativeTotals > 0) {
            for (int n7 = 0; n7 < array.length; ++n7) {
                if (this.relativeChildren[n7] > 0) {
                    array[n7] = n * this.relativeChildren[n7] / this.relativeTotals;
                }
            }
        }
        else if (n > 0) {
            float n3 = (float)(i - n);
            float[] array2 = new float[array.length];
            for (int l = 0; l < array.length; ++l) {
                array2[l] = (float)array[l] / n3 * 100.0f;
                array[l] = (int)((float)i * array2[l] / 100.0f);
                i -= array[l];
            }
            int n4 = 0;
            while (i != 0) {
                if (i < 0) {
                    int n6 = n4++;
                    --array[n6];
                    ++i;
                }
                else {
                    int n5 = n4++;
                    ++array[n5];
                    --i;
                }
                if (n4 != array.length) {
                    continue;
                }
                n4 = 0;
            }
        }
        return;
    }
    
    private int parseDigits(String s) {
        int n = 0;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (Character.isDigit((char)char1)) {
                n = n * 10 + Character.digit((char)char1, 10);
            }
        }
        return n;
    }
}
